// -*- tab-width:4 -*-
/**@file
   変換ルーチンのテストメソッドの実装。
**/

#include "ALUSConvJP.hpp"

namespace al {

int CUSConvJP::CodeConvTest()
{
    const char *test_string_utf8 = "これはテスト用文字列です。\nThis is a string for test. \\//!\"#$%";

    class Test {
        std::string		m_str[2];
        u16_string		m_str16[2];
        u32_string		m_str32[2];
        int             m_i;
        int             m_o;
        int             m_test_phase;
    public:
        Test() { Reset(); }
        void Reset() { m_i =0; m_o =1, m_test_phase=0; }
        void Proceed() { std::swap(m_i,m_o); ++m_test_phase; }
        void SetIstr(const std::string& str) { m_str[m_i] = str; }
        const std::string&	Istr() { return m_str[m_i]; }
        std::string&		Ostr() { return m_str[m_o]; }
        const u16_string&	Istr16() { return m_str16[m_i]; }
        u16_string&			Ostr16() { return m_str16[m_o]; }
        const u32_string&	Istr32() { return m_str32[m_i]; }
        u32_string&			Ostr32() { return m_str32[m_o]; }
        int TestPhase() { return m_test_phase; }
    } t;

    t.Proceed();	/* t.m_test_phase = 1 */
    do {

        /* utf-8 -> other -> utf-8 */
        t.SetIstr(test_string_utf8);
        if( !CodeConvUtf8ToSJIS(t.Istr(), t.Ostr()) ) break;
        t.Proceed();
        if( !CodeConvSJISToUtf8(t.Istr(), t.Ostr()) ) break;
        if( strcmp(test_string_utf8, t.Ostr().c_str()) != 0 ) break;
        t.Proceed();

        t.SetIstr(test_string_utf8);
        if( !CodeConvUtf8ToEucJp(t.Istr(), t.Ostr()) ) break;
        t.Proceed();
        if( !CodeConvEucJpToUtf8(t.Istr(), t.Ostr()) ) break;
        if( strcmp(test_string_utf8, t.Ostr().c_str()) != 0 ) break;
        t.Proceed();

        t.SetIstr(test_string_utf8);
        if( !CodeConvUtf8ToUtf32(t.Istr(), t.Ostr32()) ) break;
        t.Proceed();
        if( !CodeConvUtf32ToUtf8(t.Istr32(), t.Ostr()) ) break;
        if( strcmp(test_string_utf8, t.Ostr().c_str()) != 0 ) break;
        t.Proceed();

        /* utf-8 -> utf-16 -> other -> utf-8 */
        
        t.SetIstr(test_string_utf8);
        if( !CodeConvUtf8ToUtf16(t.Istr(), t.Ostr16()) ) break;
        t.Proceed();
        if( !CodeConvUtf16ToSJIS(t.Istr16(), t.Ostr()) ) break;
        t.Proceed();
        if( !CodeConvSJISToUtf8(t.Istr(), t.Ostr()) ) break;
        if( strcmp(test_string_utf8, t.Ostr().c_str()) != 0 ) break;
        t.Proceed();

        t.SetIstr(test_string_utf8);
        if( !CodeConvUtf8ToUtf16(t.Istr(), t.Ostr16()) ) break;
        t.Proceed();
        if( !CodeConvUtf16ToEucJp(t.Istr16(), t.Ostr()) ) break;
        t.Proceed();
        if( !CodeConvEucJpToUtf8(t.Istr(), t.Ostr()) ) break;
        if( strcmp(test_string_utf8, t.Ostr().c_str()) != 0 ) break;
        t.Proceed();

        t.SetIstr(test_string_utf8);
        if( !CodeConvUtf8ToUtf16(t.Istr(), t.Ostr16()) ) break;
        t.Proceed();
        if( !CodeConvUtf16ToUtf32(t.Istr16(), t.Ostr32()) ) break;
        t.Proceed();
        if( !CodeConvUtf32ToUtf8(t.Istr32(), t.Ostr()) ) break;
        if( strcmp(test_string_utf8, t.Ostr().c_str()) != 0 ) break;
        t.Proceed();

        /* utf-8 -> other -> utf-16 -> utf-8 */
        
        t.SetIstr(test_string_utf8);
        if( !CodeConvUtf8ToSJIS(t.Istr(), t.Ostr()) ) break;
        t.Proceed();
        if( !CodeConvSJISToUtf16(t.Istr(), t.Ostr16()) ) break;
        t.Proceed();
        if( !CodeConvUtf16ToUtf8(t.Istr16(), t.Ostr()) ) break;
        if( strcmp(test_string_utf8, t.Ostr().c_str()) != 0 ) break;
        t.Proceed();

        t.SetIstr(test_string_utf8);
        if( !CodeConvUtf8ToEucJp(t.Istr(), t.Ostr()) ) break;
        t.Proceed();
        if( !CodeConvEucJpToUtf16(t.Istr(), t.Ostr16()) ) break;
        t.Proceed();
        if( !CodeConvUtf16ToUtf8(t.Istr16(), t.Ostr()) ) break;
        if( strcmp(test_string_utf8, t.Ostr().c_str()) != 0 ) break;
        t.Proceed();

        t.SetIstr(test_string_utf8);
        if( !CodeConvUtf8ToUtf32(t.Istr(), t.Ostr32()) ) break;
        t.Proceed();
        if( !CodeConvUtf32ToUtf16(t.Istr32(), t.Ostr16()) ) break;
        t.Proceed();
        if( !CodeConvUtf16ToUtf8(t.Istr16(), t.Ostr()) ) break;
        if( strcmp(test_string_utf8, t.Ostr().c_str()) != 0 ) break;
        t.Proceed();

        /* utf-8 -> utf-32 -> other -> utf-8 */
        
        t.SetIstr(test_string_utf8);
        if( !CodeConvUtf8ToUtf32(t.Istr(), t.Ostr32()) ) break;
        t.Proceed();
        if( !CodeConvUtf32ToSJIS(t.Istr32(), t.Ostr()) ) break;
        t.Proceed();
        if( !CodeConvSJISToUtf8(t.Istr(), t.Ostr()) ) break;
        if( strcmp(test_string_utf8, t.Ostr().c_str()) != 0 ) break;
        t.Proceed();

        t.SetIstr(test_string_utf8);
        if( !CodeConvUtf8ToUtf32(t.Istr(), t.Ostr32()) ) break;
        t.Proceed();
        if( !CodeConvUtf32ToEucJp(t.Istr32(), t.Ostr()) ) break;
        t.Proceed();
        if( !CodeConvEucJpToUtf8(t.Istr(), t.Ostr()) ) break;
        if( strcmp(test_string_utf8, t.Ostr().c_str()) != 0 ) break;
        t.Proceed();

        /* utf-8 -> other -> utf-32 -> utf-8 */
        
        t.SetIstr(test_string_utf8);
        if( !CodeConvUtf8ToSJIS(t.Istr(), t.Ostr()) ) break;
        t.Proceed();
        if( !CodeConvSJISToUtf32(t.Istr(), t.Ostr32()) ) break;
        t.Proceed();
        if( !CodeConvUtf32ToUtf8(t.Istr32(), t.Ostr()) ) break;
        if( strcmp(test_string_utf8, t.Ostr().c_str()) != 0 ) break;
        t.Proceed();

        t.SetIstr(test_string_utf8);
        if( !CodeConvUtf8ToEucJp(t.Istr(), t.Ostr()) ) break;
        t.Proceed();
        if( !CodeConvEucJpToUtf32(t.Istr(), t.Ostr32()) ) break;
        t.Proceed();
        if( !CodeConvUtf32ToUtf8(t.Istr32(), t.Ostr()) ) break;
        if( strcmp(test_string_utf8, t.Ostr().c_str()) != 0 ) break;
        t.Proceed();

        t.Reset();

    } while(0);

    return t.TestPhase();
}

} /* namespace al */


